* Cesar Chalom wrote this to convert PNG, ICO, or other images to BMPs that
* don't need MSK files. http://weblogs.foxite.com/vfpimaging/archive/2008/12/15/7441.aspx

* VFP shows pure white - RGB(255,255,255) - as transparent in buttons and image
* objects. The code below first converts pure white pixels to RGB(254,254,254)
* that is visually the same, but does not become transparent, and eliminates
* the need to create a mask image (.MSK). It then converts the background color
* of the original bitmap to pure white, that will show transparent in VFP forms.

lparameters tcSourceFile, ;
	tcDestFile
local lcDestFile, ;
	loBmp as xfcBitmap of Source\System.Drawing.PRG, ;
	loRect as xfcRectangle of Source\System.Drawing.PRG, ;
	loDestBmp as xfcBitmap of Source\System.Drawing.PRG, ;
	loGfx as xfcGraphics  of Source\System.Drawing.PRG, ;
	loBorderClr as xfcColor of Source\System.Drawing.PRG, ;
	loColorMap as xfcColorMap of Source\System.Drawing.PRG, ;
	loAttr as xfcImageAttributes of Source\System.Drawing.PRG

* Use the same filename with a BMP extension if the destination file wasn't
* specified.

lcDestFile = iif(pcount() = 1, forceext(tcSourceFile, 'BMP'), tcDestFile)

* Ensure GDIPlusX is available for use.

do System.app
with _screen.System.Drawing as xfcDrawing of Source\System.PRG

* Read the image file, create a Graphics object from it, and get its
* dimensions.

	loBmp  = .Bitmap.FromFile(tcSourceFile)
	loRect = loBmp.GetBounds()

* Create a new 24-bit BMP using the dimensions of the original, and create a
* Graphics object from it.

	loDestBmp = .Bitmap.New(loBmp.Width, loBmp.Height, ;
		.Imaging.PixelFormat.Format24bppRGB)
	loGfx     = .Graphics.FromImage(loDestBmp)

* Clear the new bitmap

	loGfx.Clear(.Color.White)

* As per Craig Boyd, use these setting to enhance the smoothness and quality.

	loGfx.SmoothingMode     = .Drawing2D.SmoothingMode.HighQuality 
	loGfx.InterpolationMode = .Drawing2D.InterpolationMode.HighQualityBicubic
	loGfx.PixelOffsetMode   = .Drawing2D.PixelOffsetMode.HighQuality

* Get the color of the upper-left pixel, presuming this color is the background
* color to become transparent. In our case, this will become pure white - 
* RGB(255,255,255) - that becomes transparent when used in VFP objects.

	loBorderClr = loBmp.GetPixel(0, 0)

* Convert pure white to off-white - RGB(254,254,254) - by remapping the colors
* and drawing the image to the new BMP.

	loColorMap = .Imaging.ColorMap.New()
	loAttr     = .Imaging.ImageAttributes.New()
	loColorMap.OldColor = .Color.White
	loColorMap.NewColor = .Color.FromARGB(255, 254, 254, 254)
	loAttr.SetRemapTable(loColorMap)
	loGfx.DrawImage(loBmp, loRect, loRect, .GraphicsUnit.Pixel, loAttr)

* Convert the background color to pure white.

	loColorMap.OldColor = loBorderClr
	loColorMap.NewColor = .Color.White
	loAttr.SetRemapTable(loColorMap)
	loGfx.DrawImage(loDestBmp, loRect, loRect, .GraphicsUnit.Pixel, loAttr)

* Save the image to the specified file.

	loDestBmp.Save(lcDestFile, .Imaging.ImageFormat.Bmp)
endwith













