lparameters tcFileName, ;
	tnHeight, ;
	tnWidth, ;
	tnColor1, ;
	tnColor2
local lnColor1, ;
	lnColor2, ;
	lcSource, ;
	loSystem, ;
	lcExt, ;
	loFormat, ;
	loBmp, ;
	loRect, ;
	loGfx, ;
	lnMode, ;
	loBrush

#define cnERR_ARGUMENT_INVALID		11
	&& Function argument value, type, or count is invalid
 
* Ensure the parameters were passed.

do case
	case vartype(tcFileName) <> 'C' or empty(tcFileName) or ;
		vartype(tnHeight) <> 'N' or vartype(tnWidth) <> 'N'
		error cnERR_ARGUMENT_INVALID
		return .F.
	case not inlist(upper(justext(tcFileName)), 'PNG', 'BMP', 'GIF', 'JPG', ;
		'JPEG', 'ICO', 'TIF', 'TIFF', 'WMF')
		error cnERR_ARGUMENT_INVALID
		return
endcase

* If colors weren't passed, prompt for them.

if pcount() < 4
	lnColor1 = getcolor(rgb(255, 255, 255))
	lnColor2 = getcolor(rgb(  0, 128, 255))
else
	lnColor1 = tnColor1
	lnColor2 = tnColor2
endif pcount() < 4

* Ensure System is available.

do System.app
with _screen.System.Drawing as xfcDrawing of Source\System.PRG

* Determine the image type from the file extension.

	lcExt = upper(justext(tcFileName))
	do case
		case lcExt = 'PNG'
			loFormat = .Imaging.ImageFormat.Png
		case lcExt = 'BMP'
			loFormat = .Imaging.ImageFormat.Bmp
		case lcExt = 'GIF'
			loFormat = .Imaging.ImageFormat.Gif
		case inlist(lcExt, 'JPG', 'JPEG')
			loFormat = .Imaging.ImageFormat.Jpeg
		case lcExt = 'ICO'
			loFormat = .Imaging.ImageFormat.Icon
		case inlist(lcExt, 'TIF', 'TIFF')
			loFormat = .Imaging.ImageFormat.Tiff
		case lcExt = 'WMF'
			loFormat = .Imaging.ImageFormat.Wmf
	endcase

* Create a bitmap and a rectangle of the desired size.

	loBmp  = .Bitmap.New(tnWidth, tnHeight)
	loRect = .Rectangle.New(0, 0, tnWidth, tnHeight)
* Can also do this:
*	loRect = loBmp.GetBounds()

* Create a graphics object and clear the bitmap to white.

	loGfx = .Graphics.FromImage(loBmp)
	loGfx.Clear(.Color.White)

* Use a horizontal gradient if width > height or vertical if height > width.

	if tnWidth > tnHeight
		lnMode = .Drawing2D.LinearGradientMode.Horizontal
	else
		lnMode = .Drawing2D.LinearGradientMode.Vertical
	endif tnWidth > tnHeight
 
* Create a linear gradient brush.

	loBrush = .Drawing2D.LinearGradientBrush.New(loRect, ;
		.Color.FromRgb(lnColor1), .Color.FromRgb(lnColor2), lnMode)

* Fill the rectangle with the gradient brush.

	loGfx.FillRectangle(loBrush, loRect)

* Save the image to the specified file.

	loBmp.Save(tcFileName, loFormat)
endwith
return











