*==============================================================================
* Program:			DisableTreeView
* Purpose:			Adds a "lightbox" image to the specified form, visually
*						indicating that the specified TreeView is unavailable
* Author:			Doug Hennig, adapted from code written by Bernard Bout
* Last Revision:	08/20/2009
* Parameters:		toForm     - the form to add the lightbox to
*					toTreeView - the TreeView to disable
*					tlEnable   - .T. to reenable the TreeView
*					tnColor    - the color to use (optional: if it isn't passed,
*						grey--RGB(240, 240, 240)--is used)
* Returns:			.T.
* Environment in:	System.APP is available to be run
* Environment out:	the form has an image added to it if tlEnable is .F. and
*						the image is not visible if tlEnable is .T.
*					System.APP has been run, adding a System object to _screen
*					TreeView.Visible is the same as tlEnable
*==============================================================================

lparameters toForm, ;
	toTreeView, ;
	tlEnable, ;
	tnColor
local llImage, ;
	lnWidth, ;
	lnHeight, ;
	lnColor, ;
	lnFactor, ;
	lnRed, ;
	lnGreen, ;
	lnBlue, ;
	loClrMatrix, ;
	loBmp, ;
	loGfx, ;
	lcFile

* See if the specified form has a LightBox image.

llImage = pemstatus(toForm, 'imgLightBox', 5)
do case

* It does and we're supposed to enable the form, so hide the image.

	case tlEnable and llImage
		toForm.imgLightBox.Visible    = .F.
		toForm.imgLightBox.PictureVal = ''
		toTreeView.Visible            = .T.

* We're supposed to disable the form, so ensure GDIPlusX libraries are open and
* create the image if necessary.

	case not tlEnable
		do System.app
		if not llImage
			toForm.AddObject('imgLightBox', 'Image')
		endif not llImage

* Size the image as necessary.

		lnWidth  = toTreeView.Width
		lnHeight = toTreeView.Height
		with toForm.imgLightBox
			.Top    = toTreeView.Top
			.Left   = toTreeView.Left
			.Width  = lnWidth
			.Height = lnHeight
		endwith

* If we weren't passed a color to use, use grey.

		if vartype(tnColor) = 'N'
			lnColor = tnColor
		else
			lnColor = rgb(240, 240, 240)
		endif vartype(tnColor) = 'N'

* Get the colors we'll need.

		lnFactor = 0.90  && 0 = Dark   1 = Bright
		lnRed    = bitand(lnColor, 0x000000FF) / 255 * lnFactor
		lnGreen  = bitrshift(bitand(lnColor, 0x0000FF00), 8) / 255 * lnFactor
		lnBlue   = bitrshift(bitand(lnColor, 0x00FF0000), 16) / 255 * lnFactor

* Create a BMP of the desired color and use it as the picture for the image.

		with _Screen.System.Drawing as xfcDrawing of Source\System.PRG
			loClrMatrix = .Imaging.ColorMatrix.New( ;
				lnRed,       0,      0, 0, 0, ;
				    0, lnGreen,      0, 0, 0, ;
				    0,       0, lnBlue, 0, 0, ;
				    0,       0,      0, 1, 0, ;
				    0,       0,      0, 0, 1)
			loBmp = .Bitmap.FromScreen(toForm.imgLightBox)
			loBmp.ApplyColorMatrix(loClrMatrix)
			loGfx = .Graphics.FromImage(loBmp)
			loGfx.FillRectangle(.SolidBrush.New(.Color.FromARGB(10, 0, 0, 0)), ;
				0, 0, lnWidth, lnHeight)
			with toForm.imgLightBox
				.PictureVal = loBmp.GetPictureValFromHBitmap()
				.ZOrder(0)
				.Visible = .T.
			endwith
			toTreeView.Visible = .F.
		endwith
endcase














