* Program : FJLISTENER.PRG
* Purpose : Provides a Report Listener that allows rendering text in
*           Full Justify alignment.
* Author : Cesar Chalom and Victor Espinoza
* Code simplified by Doug Hennig
* Class based on article "Listening to a report" by Doug Hennig
* http://msdn2.microsoft.com/en-us/library/ms947682.aspx

define class FullJustifyListener as _ReportListener of ;
		home() + 'FFC\_ReportListener.VCX'
	oGDIGraphics = NULL
		&& a reference to a GDIPlusX Graphics object
	dimension aRecords[1]
		&& an array of records in the FRX

* If ListenerType hasn't already been set, set it based on whether the report
* is being printed or previewed.

	function LoadReport
		with This
			do case
				case .ListenerType <> -1
				case .CommandClauses.Preview
					.ListenerType = 1
				case .CommandClauses.OutputTo = 1
					.ListenerType = 0
			endcase
		endwith
		dodefault()
	endfunc

* Before we run the report, set things up the way we need them.

	function BeforeReport
		dodefault()
		with This

* Ensure GDIPlusX is available for use and create a graphics object.

			do System.App
			.oGDIGraphics = _screen.System.Drawing.Graphics.New()

* Switch to the FRX cursor's datasession and for every record with the "<FJ>"
* directive in the USER memo, flag in This.aRecords that we need to process it.
* Then restore the datasession.

			.SetFRXDataSession()
			dimension .aRecords[reccount(), 2]
			scan for '<FJ>' $ upper(USER)
				.aRecords[recno(), 2] = .T.
			endscan for '<FJ>' $ upper(USER)
			.ResetDataSession()
		endwith
	endfunc

* Put information from the FRX record into the appropriate row in
* This.aRecords.

	function EvaluateContents(tnFRXRecNo, toObjProperties)
		if This.aRecords[tnFRXRecno, 2]
			This.aRecords[tnFRXRecno, 1] = toObjProperties
		endif This.aRecords[tnFRXRecno, 2]
	endfunc

* Render the current report object.

	procedure Render(tnFRXRecNo, tnLeft, tnTop, tnWidth, tnHeight, ;
		tnObjectContinuationType, tcContentsToBeRendered, toGDIPlusImage)
		local loObject, ;
			loRect, ;
			loFont, ;
			lnAlpha, ;
			loBrush

* If the current object is supposed to be fully-justified, do so. Start by
* setting the GDI+ handle for our graphics object to the same one the report
* uses (the property to use depends on whether this is a successor or not).

		if This.aRecords[tnFRXRecNo, 2]
			loObject = This.aRecords[tnFRXRecNo, 1]
			This.oGDIGraphics.Handle = iif(This.IsSuccessor, ;
				This.SharedGDIPlusGraphics, This.GDIPlusGraphics)
			with _screen.system.Drawing as xfcDrawing of Source\System.PRG

* Create a GDI+ rectangle which specifies where on the surface we're drawing
* the text.

				loRect = .RectangleF.New(tnLeft, tnTop, tnWidth, tnHeight)

* Create a font object using the text object's settings.

				loFont = .Font.New(loObject.FontName, loObject.FontSize, ;
					loObject.FontStyle)

* If the text uses an opaque background, draw a rectangle in the chosen
* background color using a solid brush.

				lnAlpha = loObject.FillAlpha
				if lnAlpha <> 0
					loBrush = .SolidBrush.New(.Color.FromArgb(lnAlpha, ;
						loObject.FillRed, loObject.FillGreen, ;
						loObject.FillBlue))
					This.oGDIGraphics.FillRectangle(loBrush, tnLeft, ;
						tnTop, tnWidth, tnHeight)
				endif lnAlpha <> 0

* Create a solid brush and draw the text in full-justified mode.

				loBrush = .SolidBrush.New(.Color.FromArgb(loObject.PenAlpha, ;
					loObject.PenRed, loObject.PenGreen, loObject.PenBlue))
				This.oGDIGraphics.DrawStringJustified(loObject.Text, loFont, ;
					loBrush, loRect)
			endwith
		else

* If we're not drawing a full justified string, let VFP draw the text as usual.

			dodefault(tnFRXRecNo, tnLeft, tnTop, tnWidth, tnHeight, ;
				tnObjectContinuationType, tcContentsToBeRendered, ;
				toGDIPlusImage)
		endif This.aRecords[tnFRXRecNo, 2]

* Since we already drew the text, we don't want the default behavior to occur.

		nodefault
	endproc

* After the report is done, nuke the GDIPlusX graphics object's handle, since
* it is shared by the report's.

	function AfterReport
		This.oGDIGraphics.Handle = 0
	endfunc
enddefine














