* Sound constants.

#define SND_SYNC       0
#define SND_ASYNC      1
#define SND_NODEFAULT  2
#define SND_LOOP       8
#define SND_NOSTOP    16

* Other constants.

#define cnERR_ARGUMENT_INVALID		11
	&& Function argument value, type, or count is invalid

* Class to do screen shots.

define class ScreenShot as Custom
	oBackGround = .NULL.
		&& A reference to a white background form
	oBackGroundTLF = .NULL.
		&& A reference to a white background form
	nhWnd = 0
		&& The hWnd for the last form we used
	cSoundFile = 'camera_click.wav'
		&& The name of the WAV file to play
	lPlaySound = .T.
		&& .T. to play a sound after taking a screen shot

	procedure Init

* Create a white background form in case we're using Aero Glass.

		This.oBackground    = createobject('BackgroundForm')
		This.oBackgroundTLF = createobject('BackgroundFormTLF')

* Ensure System is available.

		do System.app

* Define some API functions we need.

		declare Sleep in Win32API integer nMilliseconds
		declare integer SetForegroundWindow in user32 integer hwnd
		declare integer sndPlaySound in WinMM.dll string lpszSoundName, integer uFlags
	endproc

	procedure Destroy
		This.oBackground = .NULL.
	endproc

* Take a screen shot from the specified form.

	procedure TakeScreenShot(toForm, tcImageFile, tlNoResize)
		local loBackground, ;
			lcExt, ;
			loFormat, ;
			loCaptureBmp

* Ensure the parameters are passed correctly.

		do case
			case type('toForm.hWnd') <> 'N'
				error cnERR_ARGUMENT_INVALID
				return
			case vartype(tcImageFile) <> 'C' or empty(tcImageFile)
				error cnERR_ARGUMENT_INVALID
				return
			case not inlist(upper(justext(tcImageFile)), 'PNG', 'BMP', 'GIF', ;
				'JPG', 'JPEG', 'ICO', 'TIF', 'TIFF', 'WMF')
				error cnERR_ARGUMENT_INVALID
				return
		endcase

* If we're not on the same form as before, adjust some things. First, see if we
* need a background window and if so, which one (in screen or top-level form).

		if toForm.hWnd <> This.nhWnd
			do case
				case toForm.ShowWindow = 2
					loBackground = This.oBackgroundTLF
				case toForm.Desktop
					loBackground = This.oBackground
			endcase

* If we're using a background window, make it visible and bring it to the top.
* Otherwise, make both background windows invisible.

			if vartype(loBackground) = 'O'
				loBackground.Visible = .T.
				SetForegroundWindow(This.oBackground.hWnd)
			else
				This.oBackground.Visible = .F.
				This.oBackgroundTLF.Visible = .F.
			endif vartype(loBackground) = 'O'

* Make the form the topmost window if it's visible or make it visible if it
* isn't. Notice that we call Show with 0 to make sure the form is modeless.

			if toForm.Visible
				SetForegroundWindow(toForm.hWnd)
			else
				toForm.Show(0)
			endif toForm.Visible

* Adjust the form to its smallest size.

			if not tlNoResize
				if toForm.MinWidth <> -1
					toForm.Width = toForm.MinWidth
				endif toForm.MinWidth <> -1
				if toForm.MinHeight <> -1
					toForm.Height = toForm.MinHeight
				endif toForm.MinHeight <> -1
			endif not tlNoResize

* Adjust the background form so it's larger than the target form so we have a
* nice white background for the target form.

			if vartype(loBackground) = 'O'
				with loBackground
					.Top    = toForm.Top    - 50
					.Left   = toForm.Left   - 50
					.Height = toForm.Height + 100
					.Width  = toForm.Width  + 100
				endwith
			endif vartype(loBackground) = 'O'

* Use Sleep to ensure all window drawing is complete.

			Sleep(1000)
		endif toForm.hWnd <> This.nhWnd

* Use DOEVENTS to ensure all window drawing is complete.

		doevents force
		with _Screen.System.Drawing as xfcDrawing of Source\System.PRG

* Determine the image type from the file extension.

			lcExt = upper(justext(tcImageFile))
			do case
				case lcExt = 'PNG'
					loFormat = .Imaging.ImageFormat.Png
				case lcExt = 'BMP'
					loFormat = .Imaging.ImageFormat.Bmp
				case lcExt = 'GIF'
					loFormat = .Imaging.ImageFormat.Gif
				case inlist(lcExt, 'JPG', 'JPEG')
					loFormat = .Imaging.ImageFormat.Jpeg
				case lcExt = 'ICO'
					loFormat = .Imaging.ImageFormat.Icon
				case inlist(lcExt, 'TIF', 'TIFF')
					loFormat = .Imaging.ImageFormat.Tiff
				case lcExt = 'WMF'
					loFormat = .Imaging.ImageFormat.Wmf
			endcase

* Take the screen shot.

			loCaptureBmp = .Bitmap.FromScreen(toForm.hWnd)

* Save the image to the specified file.

			loCaptureBmp.Save(tcImageFile, loFormat)

* Play a screen shot sound.

			if This.lPlaySound
				sndPlaySound(This.cSoundFile, SND_SYNC)
			endif This.lPlaySound
		endwith
		This.nhWnd = toForm.hWnd
		return
	endproc

* Shoot all pages of the specified pageframe in the form.

	procedure ShootAllPageFramePages(toForm, toPageFrame, tcRootName, ;
		tlNoResize)
		local lcRootName, ;
			lcExt, ;
			lnI
		lcRootName = addbs(justpath(tcRootName)) + juststem(tcRootName)
		lcExt      = '.' + justext(tcRootName)
		for lnI = 1 to toPageFrame.PageCount
			toPageFrame.ActivePage = lnI
			This.TakeScreenShot(toForm, lcRootName + ;
				transform(lnI) + lcExt, tlNoResize)
		next lnI
	endproc
enddefine

* Class to provide a background window.

define class BackgroundForm as Form
	Desktop   = .T.
	BackColor = rgb(255, 255, 255)
	Caption   = 'Screen Shot Background'
	Icon      = _screen.Icon
enddefine

* Class to provide a background window for top-level forms.

define class BackgroundFormTLF as BackgroundForm
	ShowWindow = 2
enddefine





