lparameters tcFileName, ;
	tcText, ;
	tcFontName, ;
	tnFontSize, ;
	tcStyle, ;
	tuTextColor, ;
	tuShadowColor, ;
	tnShadowAlpha, ;
	tnShadowOffset
local lcExt, ;
	loFormat, ;
	lnShadowAlpha, ;
	lnShadowOffset, ;
	loTextColor, ;
	loShadowColor, ;
	loGDI, ;
	loFont, ;
	loSize, ;
	loBmp, ;
	loGfx, ;
	loTextPath, ;
	loShadowPath

#define cnERR_ARGUMENT_INVALID		11
	&& Function argument value, type, or count is invalid

* Ensure the parameters are passed correctly.

do case
	case vartype(tcFileName) <> 'C' or empty(tcFileName)
		error cnERR_ARGUMENT_INVALID
		return
	case vartype(tcText) <> 'C' or empty(tcText)
		error cnERR_ARGUMENT_INVALID
		return
	case vartype(tcFontName) <> 'C' or empty(tcFontName) or ;
		vartype(tnFontSize) <> 'N' or not between(tnFontSize, 1, 128)
		error cnERR_ARGUMENT_INVALID
		return
	case pcount() > 4 and vartype(tcStyle) <> 'C'
		error cnERR_ARGUMENT_INVALID
		return
	case not inlist(upper(justext(tcFileName)), 'PNG', 'BMP', 'GIF', 'JPG', ;
		'JPEG', 'ICO', 'TIF', 'TIFF', 'WMF')
		error cnERR_ARGUMENT_INVALID
		return
endcase

* Ensure GDIPlusX is available for use.

do System.app
with _screen.System.Drawing as xfcDrawing of Source\System.PRG

* Determine the image type from the file extension.

	lcExt = upper(justext(tcFileName))
	do case
		case lcExt = 'PNG'
			loFormat = .Imaging.ImageFormat.Png
		case lcExt = 'BMP'
			loFormat = .Imaging.ImageFormat.Bmp
		case lcExt = 'GIF'
			loFormat = .Imaging.ImageFormat.Gif
		case inlist(lcExt, 'JPG', 'JPEG')
			loFormat = .Imaging.ImageFormat.Jpeg
		case lcExt = 'ICO'
			loFormat = .Imaging.ImageFormat.Icon
		case inlist(lcExt, 'TIF', 'TIFF')
			loFormat = .Imaging.ImageFormat.Tiff
		case lcExt = 'WMF'
			loFormat = .Imaging.ImageFormat.Wmf
	endcase

* Get the shadow alpha and offset, using defaults if they aren't passed.

	lnShadowAlpha  = iif(vartype(tnShadowAlpha)  = 'N', tnShadowAlpha, 50)
	lnShadowOffset = iif(vartype(tnShadowOffset) = 'N', tnShadowOffset, 6)

* Get the text and shadow colors from their RGB values or names. If the text
* color wasn't specified, use black. If the shadow color wasn't specified, use
* gray.

	do case
		case vartype(tuTextColor) = 'N'
			loTextColor = .Color.FromRGB(tuTextColor)
		case vartype(tuTextColor) = 'C'
			loTextColor = .Color.FromName(tuTextColor)
		otherwise
			loTextColor = .Color.Black
	endcase
	do case
		case vartype(tuShadowColor) = 'N'
			loShadowColor = .Color.FromARGB(lnShadowAlpha, ;
				tuShadowColor)
		case vartype(tuShadowColor) = 'C'
			loShadowColor   = .Color.FromName(tuShadowColor)
			loShadowColor.A = lnShadowAlpha
		otherwise
			loShadowColor   = .Color.Gray
			loShadowColor.A = lnShadowAlpha
	endcase

* Figure out the size required for the text, create an image that size, and
* get a Graphics object from it. Note that we have to round the size up so the
* image has enough room for the text.

	loGDI = newobject('SFGDIMeasureString', 'SFGDIMeasureString.PRG')
	loGDI.MeasureString(tcText, tcFontName, tnFontSize, evl(tcStyle, ''))
	loFont        = loGDI.oFont
	loSize        = loGDI.oStringSize
	loSize.Width  = ceiling(loSize.Width)
	loSize.Height = ceiling(loSize.Height)
	loBmp         = .Bitmap.New(loSize)
	loGfx         = .Graphics.FromImage(loBmp)

* Use anti-aliasing for better results.

	loGfx.SmoothingMode = .Drawing2D.SmoothingMode.AntiAlias

* Clear the background to transparent.

	loGfx.Clear(.Color.Transparent)

* Create a GraphicsPath object for the main text and add the text to it.

	loTextPath = .Drawing2D.GraphicsPath.New()
	loTextPath.AddString(tcText, .FontFamily.New(tcFontName), loFont.Style, ;
		tnFontSize, .Point.New(0, 0))
		
* Create a GraphicsPath object for the shadowed text and add the text to it.

	loShadowPath = .Drawing2D.GraphicsPath.New()
	loShadowPath.AddString(tcText, .FontFamily.New(tcFontName), loFont.Style, ;
		tnFontSize, .Point.New(lnShadowOffset, lnShadowOffset))

* Draw the shadowed text first (because the main text will overlay it), then
* draw the main text.

	loGfx.FillPath(.SolidBrush.New(loShadowColor), loShadowPath)
	loGfx.FillPath(.SolidBrush.New(loTextColor),   loTextPath)

* Save the image.

	loBmp.Save(tcFileName, loFormat)
endwith










