lparameters tcFileName, ;
	tcText, ;
	tcFontName, ;
	tnFontSize, ;
	tcStyle, ;
	tuForeColor, ;
	tuBackColor
local lcExt, ;
	loFormat, ;
	loForeColor, ;
	loBackColor, ;
	loGDI, ;
	loFont, ;
	loSize as xfcSize of Source\System.Drawing.PRG, ;
	loBmp, ;
	loGfx, ;
	loBrush, ;
	loStringFmt

#define cnERR_ARGUMENT_INVALID		11
	&& Function argument value, type, or count is invalid

* Ensure the parameters are passed correctly.

do case
	case vartype(tcFileName) <> 'C' or empty(tcFileName)
		error cnERR_ARGUMENT_INVALID
		return
	case vartype(tcText) <> 'C' or empty(tcText)
		error cnERR_ARGUMENT_INVALID
		return
	case vartype(tcFontName) <> 'C' or empty(tcFontName) or ;
		vartype(tnFontSize) <> 'N' or not between(tnFontSize, 1, 128)
		error cnERR_ARGUMENT_INVALID
		return
	case pcount() > 4 and vartype(tcStyle) <> 'C'
		error cnERR_ARGUMENT_INVALID
		return
	case not inlist(upper(justext(tcFileName)), 'PNG', 'BMP', 'GIF', 'JPG', ;
		'JPEG', 'ICO', 'TIF', 'TIFF', 'WMF')
		error cnERR_ARGUMENT_INVALID
		return
endcase

* Ensure GDIPlusX is available for use.

do System.app
with _screen.System.Drawing as xfcDrawing of Source\System.PRG

* Determine the image type from the file extension.

	lcExt = upper(justext(tcFileName))
	do case
		case lcExt = 'PNG'
			loFormat = .Imaging.ImageFormat.Png
		case lcExt = 'BMP'
			loFormat = .Imaging.ImageFormat.Bmp
		case lcExt = 'GIF'
			loFormat = .Imaging.ImageFormat.Gif
		case inlist(lcExt, 'JPG', 'JPEG')
			loFormat = .Imaging.ImageFormat.Jpeg
		case lcExt = 'ICO'
			loFormat = .Imaging.ImageFormat.Icon
		case inlist(lcExt, 'TIF', 'TIFF')
			loFormat = .Imaging.ImageFormat.Tiff
		case lcExt = 'WMF'
			loFormat = .Imaging.ImageFormat.Wmf
	endcase

* Get the foreground and background colors from their RGB values or names. If
* the foreground color wasn't specified, use black. If the background color
* wasn't specified, use white.

	do case
		case vartype(tuForeColor) = 'N'
			loForeColor = .Color.FromRGB(tuForeColor)
		case vartype(tuForeColor) = 'C'
			loForeColor = .Color.FromName(tuForeColor)
		otherwise
			loForeColor = .Color.Black
	endcase
	do case
		case vartype(tuBackColor) = 'N'
			loBackColor = .Color.FromRGB(tuBackColor)
		case vartype(tuBackColor) = 'C'
			loBackColor = .Color.FromName(tuBackColor)
		otherwise
			loBackColor = .Color.White
	endcase

* Figure out the size required for the text, create an image that size, and
* get a Graphics object from it. Note that we have to round the size up so the
* image has enough room for the text.

	loGDI = newobject('SFGDIMeasureString', 'SFGDIMeasureString.PRG')
	loGDI.MeasureString(tcText, tcFontName, tnFontSize, evl(tcStyle, ''))
	loFont        = loGDI.oFont
	loSize        = loGDI.oStringSize
	loSize.Width  = ceiling(loSize.Width)
	loSize.Height = ceiling(loSize.Height)
	loBmp         = .Bitmap.New(loSize)
	loGfx         = .Graphics.FromImage(loBmp)

* Use anti-aliasing for better results.

	loGfx.SmoothingMode = .Drawing2D.SmoothingMode.AntiAlias

* Clear the background to the desired color.

	loGfx.Clear(loBackColor)

* Create a solid brush in the desired forecolor.

	loBrush = .SolidBrush.New(loForeColor)

* Create a StringFormat object to center the text in the image.

	loStringFmt = .StringFormat.New()
	loStringFmt.Alignment = .StringAlignment.Center

* Draw the string and save the image.

	loGfx.DrawString(tcText, loFont, loBrush, loSize, loStringFmt)
	loBmp.Save(tcFileName, loFormat)
endwith













